/**
 * @file   zaurss.cpp
 * @author NOKUBI Takatsugu
 * @date   Wed Dec 17 13:50:38 2003
 * 
 * @brief  ZauRSS main window class
 * 
 * $Id$
 */

#include "zaurss.h"
#include "zrss.h"

#include <qpe/qcopenvelope_qws.h>

#include <qframe.h>
#include <qlayout.h>
#include <qtable.h>
#include <qtextbrowser.h>
#include <qaction.h>
#include <qpetoolbar.h>
#include <qpemenubar.h>
#include <qcstring.h>
#include <qurloperator.h>
#include <qstring.h>
#include <qnetworkprotocol.h>

#include "qinputdialog.h"
#include "dom/zrconf.h"

extern "C" {
#include <http_fetcher.h> // use HTTP Fetcher
}
#include <stdlib.h>
#include <string.h>

ZauRSS::ZauRSS(QWidget *parent = 0, const char *name = 0, int wFlags = 0)
  : QMainWindow(parent, name, wFlags)
{
  // initialize
  httpmode = 0;
  httpcont = NULL;
  rss = NULL;
  rconf = new ZRConf;
  seeNewRss = false;

  setName("ZauRSS");
  setCaption(tr("ZauRSS"));
  setToolBarsMovable(FALSE);

  QPEToolBar *toolBar = new QPEToolBar(this, "tool");
  toolBar->setHorizontalStretchable(TRUE);
  addToolBar(toolBar, "tool", QMainWindow::Top, TRUE);
  QPEMenuBar *menuBar = new QPEMenuBar(toolBar);
  QPopupMenu *listMenuFile = new QPopupMenu(menuBar);
  menuBar->insertItem(tr("File"), listMenuFile);
  this->setFont(QFont("lcfont", 16, QFont::Normal));
  QAction *actUpdate = new QAction("Update RSS", QString::null, 0, this, 0);
  connect(actUpdate, SIGNAL(activated()), this, SLOT(updaterss()));
  actUpdate->addTo(listMenuFile);
  QAction *actEnterrss = new QAction("Enter RSS", QString::null, 0, this, 0);
  connect(actEnterrss, SIGNAL(activated()), this, SLOT(getrss()));
  actEnterrss->addTo(listMenuFile);
  QAction *actAddrss = new QAction("Add RSS", QString::null, 0, this, 0);
  connect(actAddrss, SIGNAL(activated()), this, SLOT(addrss()));
  actAddrss->addTo(listMenuFile);
  QAction *actExit = new QAction("Exit", QString::null, 0, this, 0);
  connect(actExit, SIGNAL(activated()), this, SLOT(exitapp()));
  actExit->addTo(listMenuFile);

  topF = new QFrame(this);
  setCentralWidget(topF);
  topL = new QHBoxLayout(topF);
  topL->setSpacing(1);
  topL->setMargin(1);                    

  feeds = new QTable(0, 1, topF);
  feeds->setColumnMovingEnabled(false);
  feeds->setRowMovingEnabled(false);
  feeds->setFont(QFont("lcfont", 16, QFont::Normal));
  feeds->setLeftMargin(0); // hide horizontal header
  feeds->verticalHeader()->setLabel(0, "RDF");
  connect(feeds, SIGNAL(doubleClicked(int, int, int, const QPoint&)),
	  this, SLOT(selectFeed(int, int, int, const QPoint&)));
  topL->addWidget(feeds);

  rightF = new QFrame(topF);
  topL->addWidget(rightF);
  rightL = new QVBoxLayout(rightF);
  items = new QTable(0, 2, rightF);
  rightL->addWidget(items);
  text = new QTextBrowser(rightF);
  rightL->addWidget(text);

  items->setColumnMovingEnabled(false);
  items->setRowMovingEnabled(false);
  items->setFont(QFont("lcfont", 16, QFont::Normal));
  items->setLeftMargin(0); // hide horizontal header
  items->verticalHeader()->setLabel(0, "");
  items->verticalHeader()->setLabel(1, tr("Title"));
  items->setColumnWidth(0, 32);

  connect(items, SIGNAL(doubleClicked(int, int, int, const QPoint&)),
	  this, SLOT(doubleClicked(int, int, int, const QPoint&)));
  updateRssview();
}

ZauRSS::~ZauRSS()
{
  delete rconf;
  if (rss) delete rss;
}

void ZauRSS::updateRssview()
{
  feeds->setNumRows(rconf->rss.size());
  for (int i = 0; i < rconf->rss.size(); i ++) {
    QTableItem *vi = new QTableItem(feeds, QTableItem::Never,
				    rconf->rss.at(i)->title);
    feeds->setItem(i, 0, vi);
  }
}

void ZauRSS::exitapp()
{
  close();
}

void ZauRSS::updaterss()
{
  int pos = 0;
  pos = feeds->currentRow();
  QString url = rconf->rss.at(pos)->link;
  text->setText("starting update");
  if (readRss(url) == false) {
    text->setText("readRss failure.");
  } else {
      seeNewRss = true;
      text->setText(QString("updated.\nit has %1 items.")
      	    .arg(rss->getItems().size()));
      updateTable();
      QVector<ZRConfRssItem> *items = &(rconf->rss.at(pos)->items);
      items->resize(rss->getItems().size());
      for (int i = 0; i < rss->getItems().size(); i ++) {
	ZRConfRssItem *r = new ZRConfRssItem;
	r->title = rss->getItems().at(i)->title;
	r->link = rss->getItems().at(i)->link;
	items->insert(i, r);
      }
  }
}

void ZauRSS::addrss()
{
    QString url;
    bool ok = FALSE;
    url = QInputDialog::getText(tr("RDF URL"), tr("Enter URL"),
				QString::null, &ok, this);
    if (ok && !url.isEmpty()) {
	text->setText("starting update");
	if (readRss(url) == false) {
	    text->setText("readRss failure.");
	} else {
	    seeNewRss = true;
	    text->setText(QString("updated.\nit has %1 items.")
			  .arg(rss->getItems().size()));
	    updateTable();
	    ZRConfRss *nr = new ZRConfRss;
	    nr->id = rconf->rss.size();
	    nr->title = rss->getName();
	    nr->link = url;
	    nr->items.resize(rss->getItems().size());
	    for (int i = 0; i < rss->getItems().size(); i ++) {
	      ZRConfRssItem *r = new ZRConfRssItem;
	      r->title = rss->getItems().at(i)->title;
	      r->link = rss->getItems().at(i)->link;
	      nr->items.insert(i, r);
	    }
	    rconf->rss.resize(rconf->rss.size() + 1);
	    rconf->rss.insert(rconf->rss.size() -1, nr);
	    updateRssview();
	}
    }
}

bool ZauRSS::readRss(QString &url)
{
  if (httpcont != NULL) {
    free(httpcont);
    httpcont = NULL;
  }
  
  int ret = http_fetch(url.latin1(), &httpcont);
  if (ret < 0)
    return false;

  QString str = QString::fromUtf8(httpcont);
  if (rss != NULL)
    delete rss;
  rss = new ZRss(str);
  seeNewRss = true;

  return true;
}

bool ZauRSS::updateTable()
{
    items->setNumRows(rss->getItems().size());
    for (int i = 0; i < rss->getItems().size(); i ++) {
	QTableItem *opt = new QTableItem(items, QTableItem::Never, "o");
	QTableItem *text = new QTableItem(items, QTableItem::Never,
					  rss->getItems().at(i)->title);
	items->setItem(i, 0, opt);
	items->setItem(i, 1, text);
    }
    return true;
}

void ZauRSS::doubleClicked(int row, int, int, const QPoint&)
{
    // send qcop message to open a URL
    QCopEnvelope e("QPE/Application/netfront3", "setDocument(QString)");
    if (seeNewRss) {
      e << rss->getItems().at(row)->link;
      text->setText(QString("open:\n%1").arg(rss->getItems().at(row)->link));
    } else {
      e << rconf->rss.at(currentRssPos)->items.at(row)->link;
      text->setText(QString("open:\n%1").
		    arg(rconf->rss.at(currentRssPos)->items.at(row)->link));
    }
}

void ZauRSS::selectFeed(int row, int, int, const QPoint&)
{
  seeNewRss = false;
  currentRssPos = row;
  ZRConfRss *r = rconf->rss.at(currentRssPos);
  items->setNumRows(r->items.size());
  for (int i = 0; i < r->items.size(); i ++) {
    if (r->items.at(i)->seen) {
      QTableItem *opt = new QTableItem(items, QTableItem::Never, "x");
      items->setItem(i, 0, opt);
    } else {
      QTableItem *opt = new QTableItem(items, QTableItem::Never, "o");
      items->setItem(i, 0, opt);
    }
    QTableItem *text = new QTableItem(items, QTableItem::Never,
				      r->items.at(i)->title);
    items->setItem(i, 1, text);
  }
}

void ZauRSS::getrss()
{
    QString url;
    bool ok = FALSE;
    url = QInputDialog::getText(tr("RDF URL"), tr("Enter URL"),
				QString::null, &ok, this);
    if (ok && !url.isEmpty()) {
	text->setText("starting update");
	if (readRss(url) == false) {
	    text->setText("readRss failure.");
	} else {
	    seeNewRss = true;
	    text->setText(QString("updated.\nit has %1 items.")
			  .arg(rss->getItems().size()));
	    updateTable();
	}
    }
}
